from invenio_indexer.api import RecordIndexer
from invenio_records_rest.utils import allow_all
from invenio_search import RecordsSearch

RECORD_PID = 'pid(recid,record_class="sample.record:SampleRecord")'

RECORDS_REST_ENDPOINTS = {
    'recid': dict(
        pid_type='recid',
        pid_minter='recid',
        pid_fetcher='recid',
        default_endpoint_prefix=True,
        search_class=RecordsSearch,
        indexer_class=RecordIndexer,
        search_index='sample',
        search_type=None,
        record_serializers={
            'application/json': 'oarepo_validate:json_response',
        },
        search_serializers={
            'application/json': 'oarepo_validate:json_search',
        },
        record_loaders={
            'application/json': 'oarepo_validate:json_loader',
        },
        record_class='sample.record:SampleRecord',
        list_route='/records/',
        item_route='/records/<{0}:pid_value>'.format(RECORD_PID),
        default_media_type='application/json',
        max_result_window=10000,
        error_handlers=dict(),
        create_permission_factory_imp=allow_all,
        delete_permission_factory_imp=allow_all,
        update_permission_factory_imp=allow_all,
        read_permission_factory_imp=allow_all,
    )
}


# -*- coding: utf-8 -*-
#
# Copyright (C) 2020 CERN.
#
# invenio-app-ils is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""ILS Document APIs."""

import uuid

import requests
from crossref.restful import Works
from flask import current_app, Response, request
from invenio_base.utils import obj_or_import_string
from invenio_db import db
from invenio_indexer.api import RecordIndexer
from invenio_pidstore import current_pidstore
from invenio_pidstore.errors import PersistentIdentifierError, PIDDoesNotExistError
from invenio_pidstore.models import PersistentIdentifier, PIDStatus
from oarepo_actions.decorators import action

from .document_json_mapping import schema_mapping
from .minter import document_minter


class DocumentRecordMixin:
    """Class for document record."""
    DOI_PID_TYPE = 'doi'
    DOCUMENT_MINTER = document_minter
    DOCUMENT_INDEXER = RecordIndexer
    @classmethod
    @action(detail=False, method="post", url_path="document/<string:first_part>/<string:second_part>")
    def document_by_doi(cls, record_class, first_part=None, second_part=None, **kwargs):
        """Get metadata from DOI."""
        doi = first_part + '/' + second_part
        try:
            pid = PersistentIdentifier.get(cls.DOI_PID_TYPE, doi)
            record = cls.get_record(pid.object_uuid)
            return Response(status=302, headers={"Location": record.canonical_url})
        except PIDDoesNotExistError:
            pass

        existing_document = getMetadataFromDOI(doi)

        record_uuid = uuid.uuid4()
        data = schema_mapping(existing_document, doi)
        minter = cls.DOCUMENT_MINTER
        if hasattr(minter, '_get_current_object'):
            minter = minter._get_current_object()
        if isinstance(minter, str):
            minter = obj_or_import_string(current_pidstore.minters[minter])
        minter(record_uuid, data)
        record = cls.create(data=data, id_=record_uuid)
        indexer = cls.DOCUMENT_INDEXER()
        indexer.index(record)
        PersistentIdentifier.create(cls.DOI_PID_TYPE, doi, object_type='rec',
                                    object_uuid=record_uuid,
                                    status=PIDStatus.REGISTERED)

        db.session.commit()
        return Response(status=302, headers={"Location": record.canonical_url})

class CrossRefClient(object):
    """Class for CrossRefClient."""

    def __init__(self, accept='text/x-bibliography; style=apa', timeout=3):
        """
        # Defaults to APA biblio style.

        # Usage:
        s = CrossRefClient()
        print s.doi2apa("10.1038/nature10414")
        """
        self.headers = {'accept': accept}
        self.timeout = timeout

    def query(self, doi, q={}):
        #Get query.
        if doi.startswith("http://"):
            url = doi
        else:
            url = "http://dx.doi.org/" + doi

        r = requests.get(url, headers=self.headers)
        return r

    def doi2apa(self, doi):
        self.headers['accept'] = 'text/x-bibliography; style=apa'
        return self.query(doi).text

    def doi2turtle(self, doi):
        self.headers['accept'] = 'text/turtle'
        return self.query(doi).text

    def doi2json(self, doi):
        self.headers['accept'] = 'application/vnd.citationstyles.csl+json'
        return self.query(doi).json()

    def doi2xml(self, doi):
        self.headers['accept'] = 'application/rdf+xml'
        return self.query(doi).text



def getMetadataFromDOI(id):
    works = Works()
    metadata = works.doi(id)

    if metadata is None:
        s = CrossRefClient()
        metadata = s.doi2json(id)
    metadata.pop('id', None)
    return metadata




import os

from setuptools import find_packages, setup

packages = find_packages()

setup(
    name='sample',
    version='1.0.0',
    description=__doc__,
    long_description='Sample app',
    keywords='Sample app',
    license='MIT',
    author='Mirek Simek',
    author_email='miroslav.simek@vscht.cz',
    url='https://github.com/oarepo/invenio-records-draft',
    packages=packages,
    zip_safe=False,
    include_package_data=True,
    platforms='any',
    entry_points={
        'invenio_config.module': [
            'sample = config',
        ],
        'invenio_jsonschemas.schemas': [
            'sample = sample.jsonschemas'
        ],
        'invenio_search.mappings': [
            'sample = sample.mappings',
        ],
        'invenio_base.api_apps': [
            'sample = sample.ext:SampleExt',
            'oarepo_actions = oarepo_actions:Actions'
        ],
        'invenio_base.apps': [
            'sample = sample.ext:SampleExt',
            'oarepo_actions = oarepo_actions:Actions'
        ],
        # 'invenio_pidstore.fetchers': [
        #     'recid = sample.fetchers:sample_pid_fetcher'
        # ],
        # 'invenio_pidstore.minters': [
        #     'recid = sample.minters:sample_pid_minter'
        # ]
    },
    classifiers=[
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Development Status :: 3 - Alpha',
    ],
)



# -*- coding: utf-8 -*-
#
# Copyright (C) 2020 CESNET Prague.
#
# CIS theses repository is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.


"""JSON schemas.

JSON schemas are used to define the structure of the record data of your
instance.

The documentation of
`Invenio-JSONSchemas <https://invenio-jsonschemas.readthedocs.io/>`_
describes their usage and configuration options.
"""



from . import config


class SampleExt(object):
    def __init__(self, app=None):
        if app:
            self.init_app(app)

    def init_app(self, app):
        self.init_config(app)

    def init_config(self, app):
        for k in dir(config):
            if k.startswith('RECORDS_'):
                value = getattr(config, k)
                if isinstance(value, dict):
                    app.config.setdefault(k, {}).update(value)
                else:
                    app.config[k] = value
        print('Sample config initialized')



from invenio_pidstore.providers.recordid import RecordIdProvider


def document_minter(object_uuid, data):

    provider = RecordIdProvider.create(
        object_type='rec',
        object_uuid=object_uuid,
    )
    data['pid'] = provider.pid.pid_value
    return provider.pid


# -*- coding: utf-8 -*-
#
# Copyright (C) 2018-2020 CERN.
#
# invenio-app-ils is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Document schema for marshmallow loader."""

from flask import current_app
from invenio_records_rest.schemas import RecordMetadataSchemaJSONV1
from marshmallow import EXCLUDE, Schema, fields, pre_load, validate
from oarepo_multilingual.marshmallow import MultilingualStringV2

# from invenio_app_ils.documents.api import Document
# from invenio_app_ils.records.loaders.schemas.changed_by import (
#     ChangedBySchema, set_changed_by)
# from invenio_app_ils.records.loaders.schemas.preserve_cover_metadata import \
#     preserve_cover_metadata


class IdentifierSchema(Schema):
    """Identifier schema."""

    class Meta:
        """Meta attributes for the schema."""

        unknown = EXCLUDE

    material = fields.Str()
    scheme = fields.Str(required=True)
    value = fields.Str(required=True)


class AffiliationSchema(Schema):
    """Affiliation schema."""

    class Meta:
        """Meta attributes for the schema."""

        unknown = EXCLUDE

    identifiers = fields.List(fields.Nested(IdentifierSchema))
    name = fields.Str(required=True)


class AuthorSchema(Schema):
    """Author schema."""

    class Meta:
        """Meta attributes for the schema."""

        unknown = EXCLUDE

    affiliations = fields.List(fields.Nested(AffiliationSchema))
    alternative_names = fields.List(fields.Str())
    full_name = fields.Str(required=True)
    identifiers = fields.List(fields.Nested(IdentifierSchema))
    roles = fields.List(fields.Str())
    type = fields.Str()


class UrlSchema(Schema):
    """URL schema."""

    class Meta:
        """Meta attributes for the schema."""

        unknown = EXCLUDE

    description = MultilingualStringV2()
    value = fields.URL(required=True)


class AlternativeTitleSchema(Schema):
    """Alternative title schema."""

    class Meta:
        """Meta attributes for the schema."""

        unknown = EXCLUDE

    language = fields.Str()
    source = fields.Str()
    type = fields.Str()
    value = fields.Str(required=True)


class ConferenceInfoSchema(Schema):
    """Conference info schema."""

    class Meta:
        """Meta attributes for the schema."""

        unknown = EXCLUDE

    acronym = fields.Str()
    country = fields.Str()
    dates = fields.Str()
    identifiers = fields.List(fields.Nested(IdentifierSchema))
    place = fields.Str(required=True)
    series = fields.Str()
    title = MultilingualStringV2(required=True)
    year = fields.Int()


class CopyrightsSchema(Schema):
    """Copyright schema."""

    class Meta:
        """Meta attributes for the schema."""

        unknown = EXCLUDE

    holder = fields.Str()
    material = fields.Str()
    statement = fields.Str()
    url = fields.Str()
    year = fields.Int()


class ImprintSchema(Schema):
    """Imprint schema."""

    class Meta:
        """Meta attributes for the schema."""

        unknown = EXCLUDE

    date = fields.Str()
    place = fields.Str()
    publisher = fields.Str()
    reprint_date = fields.Str()


class InternalNoteSchema(Schema):
    """Internal note schema."""

    class Meta:
        """Meta attributes for the schema."""

        unknown = EXCLUDE

    field = fields.Str()
    user = fields.Str()
    value = fields.Str(required=True)


class KeywordSchema(Schema):
    """Keyword schema."""

    class Meta:
        """Meta attributes for the schema."""

        unknown = EXCLUDE

    source = fields.Str()
    value = fields.Str()


class SubjectSchema(Schema):
    """Subject schema."""

    class Meta:
        """Meta attributes for the schema."""

        unknown = EXCLUDE

    scheme = fields.Str(required=True)
    value = fields.Str(required=True)


class PublicationInfoSchema(Schema):
    """Publication info schema."""

    class Meta:
        """Meta attributes for the schema."""

        unknown = EXCLUDE

    artid = fields.Str()
    journal_issue = fields.Str()
    journal_title = MultilingualStringV2()
    journal_volume = fields.Str()
    note = fields.Str()
    pages = fields.Str()
    year = fields.Int()


class OpenDefinitionLicenseSchema(Schema):
    """Open definition license."""

    class Meta:
        """Meta attributes for the schema."""

        unknown = EXCLUDE

    id = fields.Str(required=True)
    maintainer = fields.Str()
    status = fields.Str()
    title = MultilingualStringV2()
    url = fields.Str()


class LicenseSchema(Schema):
    """License schema."""

    class Meta:
        """Meta attributes for the schema."""

        unknown = EXCLUDE

    internal_notes = fields.Str()
    license = fields.Nested(OpenDefinitionLicenseSchema)
    material = fields.Str()


class DocumentSchemaV1(RecordMetadataSchemaJSONV1):
    """Document schema."""

    class Meta:
        """Meta attributes for the schema."""

        unknown = EXCLUDE

    abstract = MultilingualStringV2()
    alternative_abstracts = MultilingualStringV2()
    alternative_identifiers = fields.List(fields.Nested(IdentifierSchema))
    alternative_titles = MultilingualStringV2()
    authors = fields.List(fields.Nested(AuthorSchema), required=True)
    conference_info = fields.Nested(ConferenceInfoSchema)
    cover_metadata = fields.Dict()
    curated = fields.Bool()
    document_type = fields.Str()
    edition = fields.Str()
    extensions = fields.Method('dump_extensions', 'load_extensions')
    identifiers = fields.List(fields.Nested(IdentifierSchema))
    imprint = fields.Nested(ImprintSchema)
    internal_notes = fields.List(fields.Nested(InternalNoteSchema))
    keywords = fields.List(fields.Nested(KeywordSchema))
    languages = fields.List(fields.Str())
    licenses = fields.List(fields.Nested(LicenseSchema))
    note = MultilingualStringV2()
    number_of_pages = fields.Str()
    other_authors = fields.Bool()
    publication_info = fields.List(fields.Nested(PublicationInfoSchema))
    publication_year = fields.Str(required=True)
    restricted = fields.Bool(missing=False)
    source = fields.Str()
    subjects = fields.List(fields.Nested(SubjectSchema))
    table_of_content = fields.List(fields.Str())
    tags = fields.List(fields.Str())
    title = MultilingualStringV2(required=True)
    urls = fields.List(fields.Nested(UrlSchema))

    def dump_extensions(self, obj):
        """Dumps the extensions value.

        :params obj: content of the object's 'extensions' field
        """
        ExtensionSchema = current_app.extensions["invenio-app-ils"] \
                                     .document_metadata_extensions \
                                     .to_schema()
        return ExtensionSchema().dump(obj)

    def load_extensions(self, value):
        """Loads the 'extensions' field.

        :params value: content of the input's 'extensions' field
        """
        ExtensionSchema = current_app.extensions["invenio-app-ils"] \
                                     .document_metadata_extensions \
                                     .to_schema()
        return ExtensionSchema().load(value)




# -*- coding: utf-8 -*-
#
# Copyright (C) 2020 CESNET.
#
# CIS theses repository is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Record serializers."""

from .version import __version__



# -*- coding: utf-8 -*-
#
# Copyright (C) 2019 CERN.
#
# My site is free software; you can redistribute it and/or modify it under
# the terms of the MIT License; see LICENSE file for more details.

"""Default configuration for My site.

You overwrite and set instance-specific configuration by either:

- Configuration file: ``<virtualenv prefix>/var/instance/invenio.cfg``
- Environment variables: ``APP_<variable name>``
"""

from __future__ import absolute_import, print_function

from datetime import timedelta


def _(x):
    """Identity function used to trigger string extraction."""
    return x


# Rate limiting
# =============
#: Storage for ratelimiter.
RATELIMIT_STORAGE_URL = 'redis://localhost:6379/3'

# I18N
# ====
#: Default language
BABEL_DEFAULT_LANGUAGE = 'en'
#: Default time zone
BABEL_DEFAULT_TIMEZONE = 'Europe/Zurich'
#: Other supported languages (do not include the default language in list).
I18N_LANGUAGES = [
    # ('fr', _('French'))
]

# Base templates
# ==============
#: Global base template.
BASE_TEMPLATE = 'my_site/page.html'
#: Cover page base template (used for e.g. login/sign-up).
COVER_TEMPLATE = 'invenio_theme/page_cover.html'
#: Footer base template.
FOOTER_TEMPLATE = 'invenio_theme/footer.html'
#: Header base template.
HEADER_TEMPLATE = 'invenio_theme/header.html'
#: Settings base template.
SETTINGS_TEMPLATE = 'invenio_theme/page_settings.html'

# Theme configuration
# ===================
#: Site name
THEME_SITENAME = _('My site')
#: Use default frontpage.
THEME_FRONTPAGE = True
#: Frontpage title.
THEME_FRONTPAGE_TITLE = _('My site')
#: Frontpage template.
THEME_FRONTPAGE_TEMPLATE = 'my_site/frontpage.html'

# Email configuration
# ===================
#: Email address for support.
SUPPORT_EMAIL = "info@my-site.com"
#: Disable email sending by default.
MAIL_SUPPRESS_SEND = True

# Assets
# ======
#: Static files collection method (defaults to copying files).
COLLECT_STORAGE = 'flask_collect.storage.file'

# Accounts
# ========
#: Email address used as sender of account registration emails.
SECURITY_EMAIL_SENDER = SUPPORT_EMAIL
#: Email subject for account registration emails.
SECURITY_EMAIL_SUBJECT_REGISTER = _(
    "Welcome to My site!")
#: Redis session storage URL.
ACCOUNTS_SESSION_REDIS_URL = 'redis://localhost:6379/1'
#: Enable session/user id request tracing. This feature will add X-Session-ID
#: and X-User-ID headers to HTTP response. You MUST ensure that NGINX (or other
#: proxies) removes these headers again before sending the response to the
#: client. Set to False, in case of doubt.
ACCOUNTS_USERINFO_HEADERS = True

# Celery configuration
# ====================

BROKER_URL = 'amqp://guest:guest@localhost:5672/'
#: URL of message broker for Celery (default is RabbitMQ).
CELERY_BROKER_URL = 'amqp://guest:guest@localhost:5672/'
#: URL of backend for result storage (default is Redis).
CELERY_RESULT_BACKEND = 'redis://localhost:6379/2'
#: Scheduled tasks configuration (aka cronjobs).
CELERY_BEAT_SCHEDULE = {
    'indexer': {
        'task': 'invenio_indexer.tasks.process_bulk_queue',
        'schedule': timedelta(minutes=5),
    },
    'accounts': {
        'task': 'invenio_accounts.tasks.clean_session_table',
        'schedule': timedelta(minutes=60),
    },
}

# Database
# ========
#: Database URI including user and password

# JSONSchemas
# ===========
#: Hostname used in URLs for local JSONSchemas.
JSONSCHEMAS_HOST = 'my-site.com'

# Flask configuration
# ===================
# See details on
# http://flask.pocoo.org/docs/0.12/config/#builtin-configuration-values

#: Secret key - each installation (dev, production, ...) needs a separate key.
#: It should be changed before deploying.
SECRET_KEY = 'CHANGE_ME'
#: Max upload size for form data via application/mulitpart-formdata.
MAX_CONTENT_LENGTH = 100 * 1024 * 1024  # 100 MiB
#: Sets cookie with the secure flag by default
SESSION_COOKIE_SECURE = True
#: Since HAProxy and Nginx route all requests no matter the host header
#: provided, the allowed hosts variable is set to localhost. In production it
#: should be set to the correct host and it is strongly recommended to only
#: route correct hosts to the application.
APP_ALLOWED_HOSTS = ['my-site.com', 'localhost', '127.0.0.1']

# OAI-PMH
# =======
OAISERVER_ID_PREFIX = 'oai:my-site.com:'

# Debug
# =====
# Flask-DebugToolbar is by default enabled when the application is running in
# debug mode. More configuration options are available at
# https://flask-debugtoolbar.readthedocs.io/en/latest/#configuration

#: Switches off incept of redirects by Flask-DebugToolbar.
DEBUG_TB_INTERCEPT_REDIRECTS = False

RECORDS_REST_ENDPOINTS = {}



# -*- coding: utf-8 -*-
"""Setup module for flask taxonomy."""
import os

from setuptools import setup

readme = open('README.md').read()
history = open('CHANGES.md').read()
OAREPO_VERSION = os.environ.get('OAREPO_VERSION', '3.3.0')


install_requires = [
    #'oarepo-records-draft',
    #'oarepo-actions',
    #'oarepo-validate',
    # 'coverage',
    'oarepo-mapping-includes',
    'marshmallow',
    # 'oarepo-multilingual',
    'flask',
    'crossrefapi',
    'deepmerge',
    'langdetect'
]

tests_require = [
    'oarepo-records-draft',
    'oarepo-actions',
    'oarepo-validate',
    'coverage',
    'oarepo-multilingual',
    'oarepo-mapping-includes',
    'invenio-search',
    'crossrefapi',
    'deepmerge',
    'langdetect',
]

extras_require = {
    'tests': [
    'oarepo[tests]~={version}'.format(
            version=OAREPO_VERSION),
        *tests_require,
        ],
    'tests-es7': [
    'oarepo[tests-es7]~={version}'.format(
            version=OAREPO_VERSION),
        *tests_require,
        ],
}

setup_requires = [
    'pytest-runner>=2.7',
]

g = {}
with open(os.path.join('oarepo_documents', 'version.py'), 'rt') as fp:
    exec(fp.read(), g)
    version = g['__version__']

setup(
    name="oarepo_documents",
    version=version,
    url="https://github.com/oarepo/oarepo-dc",
    license="MIT",
    author="Alzbeta Pokorna",
    author_email="alzbeta.pokorna@cesnet.cz",
    description="OARepo rdm records data model",
    long_description=readme + '\n\n' + history,
    long_description_content_type='text/markdown',
    zip_safe=False,
    packages=['oarepo_documents'],
    entry_points={
        'oarepo_mapping_includes':[
            'oarepo_documents = oarepo_documents.included_mappings'
        ],
        #'invenio_search.mappings': [
        #    'documents= oarepo_documents.mappings',
        #],
        # 'invenio_search.mappings': [
        #     'test_mapping = test_mapping.mappings'
        # ],
        'invenio_jsonschemas.schemas': [
            'oarepo_documents = oarepo_documents.jsonschemas',
        ],
        'invenio_base.apps': [
            #'document = oarepo_documents.DocumentRecordMixin',
            'oarepo_actions = oarepo_actions:Actions'
        ],
    },
    include_package_data=True,
    setup_requires=setup_requires,
    extras_require=extras_require,
    install_requires=install_requires,
    tests_require=tests_require,
    platforms='any',
    classifiers=[
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Development Status :: 4 - Beta',
    ],
)



from marshmallow import fields, validate

#from oarepo_invenio_model.marshmallow import InvenioRecordMetadataSchemaV1Mixin
from oarepo_documents.marshmallow.document import DocumentSchemaV1


class SampleSchemaV1(DocumentSchemaV1):
    extra_property = fields.String(validate=validate.Length(min=5), required=False)



from invenio_records.api import Record
from oarepo_actions.decorators import action
from oarepo_validate import MarshmallowValidatedRecordMixin, SchemaKeepingRecordMixin

from oarepo_documents.api import DocumentRecordMixin

from .constants import SAMPLE_ALLOWED_SCHEMAS, SAMPLE_PREFERRED_SCHEMA
from .marshmallow import SampleSchemaV1


class SampleRecord(SchemaKeepingRecordMixin,
                   MarshmallowValidatedRecordMixin,
                   DocumentRecordMixin,
                   Record):
    ALLOWED_SCHEMAS = SAMPLE_ALLOWED_SCHEMAS
    PREFERRED_SCHEMA = SAMPLE_PREFERRED_SCHEMA
    MARSHMALLOW_SCHEMA = SampleSchemaV1

    @property
    def canonical_url(self):
        return f'https://localhost:5000/records/{self["pid"]}'

    def validate(self, **kwargs):
        return super().validate(**kwargs)



from deepmerge import always_merger
from langdetect import detect, detect_langs


def try_name(nlist,record, default=None):
    for name in nlist:
        try:
            return record[name]
        except:
            continue
    else:
        return default

def schema_mapping(existing_record, doi):
    data = {}

    #abstract - multilingual
    abstract_value =  try_name(nlist = ['abstract', 'abstracts'], record =existing_record)
    if abstract_value != None:
        probability = detect_langs(abstract_value.lower())
        if probability[0].prob >= 0.99999:
            abstract_language = probability[0].lang
        else:
            abstract_language = '_'  # unknown language

        abstract = {abstract_language: abstract_value}
        always_merger.merge(data, {'abstract': abstract})

    #alternative_abstracts - array of strings
    alternative_abstract = try_name(['alternative_abstract', 'alternative_abstracts'], existing_record)
    if alternative_abstract != None:
        if(type(alternative_abstract) is list):
            is_string = True
            for a in alternative_abstract:
                if type(a) is not str:
                    is_string = False
                    break
            if is_string:
                aa = {}
                for a in alternative_abstract:
                    probability = detect_langs(a.lower())
                    if probability[0].prob >= 0.99999:
                        abstract_language = probability[0].lang
                    else:
                        abstract_language = '_'  # unknown language

                    abstract = {abstract_language: a}
                    aa = always_merger.merge(aa, abstract)

                always_merger.merge(data, {'alternative_abstracts': aa})

    #alternative_identifiers
    always_merger.merge(data, {"alternative_identifiers": [{"scheme": "DOI", "value": doi}]})

    #alternative_titles
    alternative_titles = try_name(['alternative_title', 'alternative_titles', 'shor-contrainer-title'], existing_record)
    if alternative_titles != None:
        if (type(alternative_titles) is list):
            is_string = True
            for a in alternative_titles:
                if type(a) is not str:
                    is_string = False
                    break
            if is_string:
                aa = {}
                for a in alternative_titles:
                    probability = detect_langs(a.lower())
                    if probability[0].prob >= 0.99999:
                        title_language = probability[0].lang
                    else:
                        title_language = '_'  # unknown language

                    title = {title_language: a}
                    aa = always_merger.merge(aa, title)
                always_merger.merge(data, {'alternative_titles': aa})

    #authors
    authors_array = try_name(nlist=['authors', 'author', 'contributor', 'contributors'], record=existing_record)
    if authors_array == None:
        always_merger.merge(data, {'authors': [{"full_name": "Various authors"}]}) #default
    else:
        if(type(authors_array) is list):
            authors_data = []
            for author in authors_array:
                auth_data = {}
                if 'ORCID' in author:
                    always_merger.merge(auth_data, {"identifiers": [{"scheme": "ORCID", "value": author["ORCID"]}]})
                if 'alternative_names' in author and type(author['alternative_names']) is list:
                    always_merger.merge(auth_data, {"alternative_names": author['alternative_names']})
                if 'roles' in author and type(author['roles']) is list:
                    always_merger.merge(auth_data, {"roles": author['roles']})
                if 'type' in author and type(author['type']) is str:
                    always_merger.merge(auth_data, {"type": author['type']})
                #affiliation /affiliations
                full_name = try_name(nlist=['full_name', 'name', 'fullname', 'literal'], record=author)
                if full_name != None:
                    always_merger.merge(auth_data, {"full_name": full_name})
                    authors_data.append(auth_data)
                    continue
                given = try_name(nlist=['given', 'first', 'first_name'], record=author)
                family = try_name(nlist=['family', 'family_name', 'second_name'], record=author)
                if(given == None or family == None):
                    always_merger.merge(auth_data, {"full_name": "unknown"})
                    authors_data.append(auth_data)
                    continue
                else:
                    full_name = given + " " + family
                    always_merger.merge(auth_data, {"full_name": full_name})
                    authors_data.append(auth_data)
                    continue

            always_merger.merge(data, {'authors': authors_data})

    # document_type
    doctype = try_name(nlist=['document_type', 'type'], record=existing_record)
    if doctype == None:
        always_merger.merge(data, {'document_type': "unknown"})  # default
    else:
        always_merger.merge(data, {'document_type': doctype})

    #publication_year
    publication_year = try_name(nlist=['publication_year', 'issued'], record=existing_record)

    if publication_year != None and type(publication_year) is str and len(publication_year['data-part'][0]) == 4:
        always_merger.merge(data, {'publication_year': publication_year})
    elif publication_year != None and type(publication_year) is dict:
        if 'date-parts' in publication_year.keys():
            if len(str(publication_year['date-parts'][0][0])) == 4:
                always_merger.merge(data, {'publication_year': str(publication_year['date-parts'][0][0])})
            else:
                always_merger.merge(data, {'publication_year': "unknown"})
        else:
            always_merger.merge(data, {'publication_year': "unknown"})
    else:
        always_merger.merge(data, {'publication_year': "unknown"})

    # title - multilingual --- mezivalidacky!
    title_value = try_name(nlist=['title', 'titles'], record=existing_record)
    if title_value != None:
        if type(title_value) is list:
            title_value= title_value[0]
        probability = detect_langs(title_value.lower())
        if probability[0].prob >= 0.99999:
            title_language = probability[0].lang
        else:
            title_language = '_'  # unknown language

        title = {title_language: title_value}
        always_merger.merge(data, {'title': title})
    else:
        always_merger.merge(data, {'title': "unknown"}) #default

    # copyright
    # copyrights = try_name(nlist=['copyright', 'copyrights'], record=existing_record)
    # if copyrights != None and type(copyrights) is str:
    #     always_merger.merge(data, {'copyrigts': [{"statement": copyrights}]})

    # urls
    urls = try_name(nlist=['url', 'urls', 'URL', 'URLs'], record=existing_record)
    if urls != None and type(urls) is str:
        always_merger.merge(data, {'urls': [{"value": urls}]})

    # keywords
    keywords = try_name(nlist=['keywords', 'keyword', 'categories'], record=existing_record)
    if keywords != None and type(keywords) is list:
        keywords_array = []
        if len(keywords) == 1:
            keys = keywords[0].split()
        else:
            keys = keywords
        i = 0
        while (i < len(keys)):
            lowerCounter = 0
            for key in keys:
                if key[0].islower():
                    lowerCounter = lowerCounter + 1
                else:
                    break
            if lowerCounter == len(keywords):  # if everything in lowercase
                for key in keys:
                    key_json = {"value": key}
                    keywords_array.append(key_json)
                break
            key = keys[i]
            numberOfKeysTogether = 0
            for j in range(i + 1, len(keys)):
                if keys[j][0].islower():
                    key = key + ' ' + keys[j]
                    numberOfKeysTogether = numberOfKeysTogether + 1
                else:
                    break
            i = i + 1 + numberOfKeysTogether

            key_json = {"value": key}
            keywords_array.append(key_json)

        always_merger.merge(data, {'keywords': keywords_array})
    return data